;; Generated package description from gnosis.el  -*- no-byte-compile: t -*-
(define-package "gnosis" "0.4.6" "Spaced Repetition System" '((emacs "27.2") (emacsql "4.0.1") (compat "29.1.4.2") (transient "0.7.2")) :commit "f4050f4f27e1ac0eab25168b0849017afd04aa31" :authors '(("Thanos Apollo" . "public@thanosapollo.org")) :maintainer '("Thanos Apollo" . "public@thanosapollo.org") :keywords '("extensions") :url "https://thanosapollo.org/projects/gnosis")
